@@ -10,6 +10,15 @@ use Plack::Request;
 
 Plack::App::RDF::LinkedData - A Plack application for running RDF::LinkedData
 
+=head1 VERSION
+
+Version 0.75_01
+
+=cut
+
+ our $VERSION = '0.75_01';
+
+
 =head1 SYNOPSIS
 
   my $linkeddata = Plack::App::RDF::LinkedData->new();
@@ -133,7 +142,10 @@ is are longer examples in the distribtion, which is used to run
 tests. In the config file, there is a C<store> parameter, which must
 contain the L<RDF::Trine::Store> config hashref. It may also have a
 C<base_uri> URI and a C<namespace> hashref which may contain prefix -
-URI mappings to be used in serializations.
+URI mappings to be used in serializations. Certain namespace, namely
+RDF, VoID, Hydra, DC Terms and XML Schema are added by the module and
+do not need to be declared.
+
 
 Note that this is a server that can only serve URIs of hosts you
 control, it is not a general purpose Linked Data manipulation tool,
@@ -303,9 +315,13 @@ sub prepare_app {
 	$self->{linkeddata} = RDF::LinkedData->new(store => $config->{store},
 															 endpoint_config => $config->{endpoint},
 															 void_config => $config->{void},
+															 fragments_config => $config->{fragments},
 															 base_uri => $config->{base_uri}
 															);
 	$self->{linkeddata}->namespaces(URI::NamespaceMap->new($config->{namespaces})) if ($config->{namespaces});
+	# Ensure that certain namespaces are always declared
+	$self->{linkeddata}->guess_namespaces('rdf', 'dct', 'xsd', 'void');
+	$self->{linkeddata}->add_namespace_mapping(hydra => 'http://www.w3.org/ns/hydra/core#');
 }
 
 sub call {
@@ -344,7 +360,7 @@ Kjetil Kjernsmo, C<< <kjetilk@cpan.org> >>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2010,2011,2012,2013,2014 Kjetil Kjernsmo
+Copyright 2010, 2011, 2012, 2013, 2014, 2015, 2016 Kjetil Kjernsmo
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
