@@ -1,13 +1,13 @@
 package RDF::LinkedData;
 
+use strict;
+use warnings;
 use Moo;
 use namespace::autoclean;
 use Types::Standard qw(InstanceOf Str Bool Maybe Int HashRef);
 
 use RDF::Trine qw[iri literal blank statement variable];
 use RDF::Trine::Serializer;
-use RDF::Trine::Namespace;
-use Log::Log4perl qw(:easy);
 use Plack::Response;
 use RDF::Helper::Properties;
 use URI::NamespaceMap;
@@ -22,25 +22,8 @@ use Digest::MD5 ('md5_base64');
 use Carp;
 use Try::Tiny;
 use List::Util qw(any);
-use Log::Log4perl;
-use Log::Log4perl ':easy';
-use Log::Contextual qw( :log ), -package_logger => Log::Log4perl->get_logger;
-
-BEGIN {
-	if ($ENV{TEST_VERBOSE}) {
-		Log::Log4perl->easy_init( { level   => $TRACE,
-											 category => 'RDF.LinkedData' 
-										  } );
-	} else {
-		Log::Log4perl->easy_init( { level   => $FATAL,
-											 category => 'RDF.LinkedData' 
-										  } );
-	}
-	use Log::Contextual -logger => Log::Log4perl->get_logger;
-}
-
-
 
+with 'MooX::Log::Any';
 
 =head1 NAME
 
@@ -48,11 +31,11 @@ RDF::LinkedData - A Linked Data server implementation
 
 =head1 VERSION
 
-Version 0.69_10
+Version 0.75_01
 
 =cut
 
- our $VERSION = '0.69_10';
+ our $VERSION = '0.75_01';
 
 
 =head1 SYNOPSIS
@@ -129,7 +112,7 @@ sub BUILD {
 	}
 
  	if ($self->has_endpoint_config) {
-		log_debug {'Endpoint config found with parameters: ' . Dumper($self->endpoint_config) };
+		$self->log->debug('Endpoint config found with parameters: ' . Dumper($self->endpoint_config) );
 
 		unless (can_load( modules => { 'RDF::Endpoint' => 0.03 })) {
 			throw Error -text => "RDF::Endpoint not installed. Please install or remove its configuration.";
@@ -141,11 +124,11 @@ sub BUILD {
 
 		$self->endpoint(RDF::Endpoint->new($self->model, $self->endpoint_config));
  	} else {
-		log_info {'No endpoint config found'};
+		$self->log->info('No endpoint config found');
 	}
 
  	if ($self->has_void_config) {
-		log_debug {'VoID config found with parameters: ' . Dumper($self->void_config) };
+		$self->log->debug('VoID config found with parameters: ' . Dumper($self->void_config) );
 
 		unless (can_load( modules => { 'RDF::Generator::Void' => 0.04 })) {
 			throw Error -text => "RDF::Generator::Void not installed. Please install or remove its configuration.";
@@ -158,10 +141,10 @@ sub BUILD {
 														  dataset_uri => $dataset_uri,
 														  namespaces_as_vocabularies => $self->void_config->{namespaces_as_vocabularies}));
 		if ($self->has_fragments) {
-			log_debug {'Triple Pattern Fragments config found with parameters: ' . Dumper($self->fragments_config) };
+			$self->log->debug('Triple Pattern Fragments config found with parameters: ' . Dumper($self->fragments_config) );
 		}
  	} else {
-		log_info {'No VoID config found'};
+		$self->log->info('No VoID config found');
 	}
 }
 
@@ -227,7 +210,7 @@ has hypermedia => (is => 'ro', isa => Bool, default => 1);
 
 has namespaces_as_vocabularies => (is => 'ro', isa => Bool, default => 1);
 
-has endpoint_config => (is => 'rw',	isa=>Maybe[HashRef], predicate => 'has_endpoint_config');
+has endpoint_config => (is => 'rw', isa=>Maybe[HashRef], predicate => 'has_endpoint_config');
 
 has void_config => (is => 'rw', isa=>Maybe[HashRef], predicate => 'has_void_config');
 
@@ -257,9 +240,10 @@ Returns or sets the last Etag of so that changes to the model can be detected.
 has last_etag => ( is => 'rw', isa => Str, predicate => 'has_last_etag');
 
 
-=item namespaces ( { skos => 'http://www.w3.org/2004/02/skos/core#', dct => 'http://purl.org/dc/terms/' } )
+=item namespaces ( $namespace_map )
 
-Gets or sets the namespaces that some serializers use for pretty-printing.
+Gets or sets the namespaces that some serializers use for
+pretty-printing. Should be handed a L<URI::NamespaceMap> object.
 
 =cut
 
@@ -269,13 +253,14 @@ has 'namespaces' => (is => 'rw',
 							lazy => 1,
 							handles => {
 											'add_namespace_mapping' => 'add_mapping',
+											'guess_namespaces' => 'guess_and_add',
 											'list_namespaces' => 'list_namespaces'
 										  });
 
 
 sub _build_namespaces {
-  my ($self, $ns_hash) = @_;
-  return $ns_hash || URI::NamespaceMap->new({ rdf => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#' });
+  my $self = shift;
+  return shift || URI::NamespaceMap->new();
 }
 
 # Just a temporary compatibility hack
@@ -349,8 +334,9 @@ sub response {
 			}
 		}
 
-		log_debug {'Getting fragment with this selector ' . Dumper(\%statement) };
-		return _client_error($response, 'Returning the whole database not allowed') unless any { defined } values(%statement);
+		$self->log->debug('Getting fragment with this selector ' . Dumper(\%statement) );
+		return _client_error($response, 'Returning the whole database not allowed') 
+				unless $self->fragments_config->{allow_dump_dataset} || any { defined } values(%statement);
 		my $output_model = $self->_common_fragments_control;
 
 		my $iterator = $self->model->get_statements($statement{subject}, $statement{predicate}, $statement{object});
@@ -361,22 +347,25 @@ sub response {
 			# TODO: Paging goes here
 			$output_model->add_statement($st);
 		}
-		$self->add_namespace_mapping(void => 'http://rdfs.org/ns/void#');
-		$self->add_namespace_mapping(hydra => 'http://www.w3.org/ns/hydra/core#');
 		my $cl = literal($counter, undef, 'http://www.w3.org/2001/XMLSchema#integer');
-		my $void = RDF::Trine::Namespace->new('http://rdfs.org/ns/void#');
+		$self->guess_namespaces('void');
+		my $void = $self->namespaces->void;
 		$output_model->add_statement(statement(iri($uri), 
-															$void->triples,
+															iri($void->triples),
 															$cl));
 		$output_model->add_statement(statement(iri($uri), 
 															iri('http://www.w3.org/ns/hydra/core#totalItems'),
 															$cl));
 		$output_model->add_statement(statement(iri($uri),
 															iri('http://www.w3.org/1999/02/22-rdf-syntax-ns#type'),
-															$void->Dataset));
+															iri($void->Dataset)));
 		$output_model->add_statement(statement($self->void->dataset_uri,
-															$void->subset,
+															iri($void->subset),
 															iri($uri)));
+		$output_model->add_statement(statement(iri($uri),
+															iri('http://purl.org/dc/terms/source'),
+															$self->void->dataset_uri
+															));
 		$output_model->end_bulk_ops;
 		my ($ct, $s);
 		try {
@@ -392,11 +381,11 @@ sub response {
 
 		$response->status(200);
 		$response->headers->header('Vary' => join(", ", qw(Accept)));
-		if (defined($self->last_etag)) {
-		  $response->headers->header('ETag' => '"' . $self->last_etag . '"');
+		if ($self->has_last_etag) {
+			$response->headers->header('ETag' => '"' . md5_base64($self->last_etag . $ct) . '"');
 		}
 		my $body = $s->serialize_model_to_string($output_model);
-		log_trace { "Fragment message body is $body" };
+		$self->log->trace("Fragment message body is $body" );
 		$response->headers->content_type($ct);
 		$response->body(encode_utf8($body));
 		return $response;
@@ -410,7 +399,7 @@ sub response {
 	my $type = $self->type;
 	$self->type('');
 	my $node = $self->my_node($uri);
-	log_info{"Try rendering '$type' page for subject node: " . $node->as_string};
+	$self->log->info("Try rendering '$type' page for subject node: " . $node->as_string);
 	if ($self->count($node) > 0) {
 		if ($type) {
 			my $preds = $self->helper_properties;
@@ -422,15 +411,15 @@ sub response {
 				return $response;
 			}
 
-			log_debug {"Will render '$type' page " };
+			$self->log->debug("Will render '$type' page " );
 			if ($headers_in->can('header') && $headers_in->header('Accept')) {
-				log_debug {'Found Accept header: ' . $headers_in->header('Accept') };
+				$self->log->debug('Found Accept header: ' . $headers_in->header('Accept') );
 			} else {
 				$headers_in->header('Accept' => 'application/rdf+xml');
 				if ($headers_in->header('Accept')) {
-					log_warn { 'Setting Accept header: ' . $headers_in->header('Accept') };
+					$self->log->warn('Setting Accept header: ' . $headers_in->header('Accept') );
 				} else {
-					log_warn { 'No content type header can be set' };
+					$self->log->warn('No content type header can be set' );
 				}
 			}
 			$response->status(200);
@@ -450,7 +439,7 @@ sub response {
 				my $preds = $self->helper_properties;
 				$newurl = $preds->page($node);
 			}
-			log_debug {'Will do a 303 redirect to ' . $newurl };
+			$self->log->debug('Will do a 303 redirect to ' . $newurl );
 			$response->headers->header('Location' => $newurl);
 			$response->headers->header('Vary' => join(", ", qw(Accept)));
 		}
@@ -513,7 +502,7 @@ get a URI object containing the full URI of the node.
 
 sub my_node {
 	my ($self, $iri) = @_;
-	log_info { "Subject URI to be used: $iri" };
+	$self->log->info("Subject URI to be used: $iri" );
 	return RDF::Trine::Node::Resource->new( $iri );
 }
 
@@ -584,7 +573,7 @@ sub _content {
 			$iter = $iter->concat($hmmodel->as_stream);
 		}
 		$output{body} = $s->serialize_iterator_to_string ( $iter );
-		log_trace { "Message body is $output{body}" };
+		$self->log->trace("Message body is $output{body}" );
 
 	} else {
 		$self->{_type} = 'page';
@@ -645,7 +634,7 @@ sub _negotiate {
 																					'application/xhtml+xml' => 'xhtml'
 																				  }
 																	);
-		log_debug { "Got $ct content type" };
+		$self->log->debug("Got $ct content type" );
 		1;
 	} or do {
 		my $response = Plack::Response->new;
@@ -723,7 +712,6 @@ sub _void_content {
 		}
 
 		if ($self->has_fragments) {
-			$self->add_namespace_mapping(hydra => 'http://www.w3.org/ns/hydra/core#');
 			$self->_common_fragments_control($self->_voidmodel);
 		}
 
@@ -769,46 +757,60 @@ has _last_extvoid_mtime => (is => 'rw', isa => Int);
 sub _common_fragments_control {
 	my $self = shift;
 	my $model = shift || RDF::Trine::Model->temporary_model;
-	my $void = RDF::Trine::Namespace->new('http://rdfs.org/ns/void#');
-	my $xsd  = RDF::Trine::Namespace->new('http://www.w3.org/2001/XMLSchema#');
-	my $hydra = RDF::Trine::Namespace->new('http://www.w3.org/ns/hydra/core#');
-	my $rdf = RDF::Trine::Namespace->new('http://www.w3.org/1999/02/22-rdf-syntax-ns#');
+	$self->guess_namespaces('rdf', 'void');
+	$self->add_namespace_mapping(hydra => 'http://www.w3.org/ns/hydra/core#');
+	my $void = $self->namespaces->void;
+	my $hydra = $self->namespaces->hydra;
+	my $rdf = $self->namespaces->rdf;
 	$model->begin_bulk_ops;
 	my $void_subject = $self->void->dataset_uri;
 	$model->add_statement(statement($void_subject,
-											  $rdf->type,
-											  $hydra->Collection));
+											  iri($rdf->type),
+											  iri($hydra->Collection)));
 	$model->add_statement(statement($void_subject,
-											  $rdf->type,
-											  $void->Dataset));
+											  iri($rdf->type),
+											  iri($void->Dataset)));
 	$model->add_statement(statement($void_subject,
-											  $hydra->search,
+											  iri($hydra->search),
 											  blank('template')));
 	$model->add_statement(statement($void_subject,
-								 $void->uriLookupEndpoint,
+								 iri($void->uriLookupEndpoint),
 								 literal($self->base_uri . $self->fragments_config->{fragments_path}
 											. '{?subject,predicate,object}')));
 	$model->add_statement(statement(blank('template'),
-								 $hydra->template,
+								 iri($hydra->template),
 								 literal($self->base_uri . $self->fragments_config->{fragments_path}
 											. '{?subject,predicate,object}')));
+	
 	$model->add_statement(statement(blank('template'),
-								 $hydra->property,
-								 $rdf->subject));
+								  iri($hydra->mapping),
+								  blank('subject')));
 	$model->add_statement(statement(blank('template'),
-								 $hydra->variable,
+								  iri($hydra->mapping),
+								  blank('predicate')));
+	$model->add_statement(statement(blank('template'),
+								  iri($hydra->mapping),
+								  blank('object')));
+
+	$model->add_statement(statement(blank('subject'),
+								 iri($hydra->property),
+								 iri($rdf->subject)));
+	$model->add_statement(statement(blank('subject'),
+								 iri($hydra->variable),
 								 literal('subject')));
-	$model->add_statement(statement(blank('template'),
-								 $hydra->property,
-								 $rdf->predicate));
-	$model->add_statement(statement(blank('template'),
-								 $hydra->variable,
+
+	$model->add_statement(statement(blank('predicate'),
+								 iri($hydra->property),
+								 iri($rdf->predicate)));
+	$model->add_statement(statement(blank('predicate'),
+								 iri($hydra->variable),
 								 literal('predicate')));
-	$model->add_statement(statement(blank('template'),
-								 $hydra->property,
-								 $rdf->object));
-	$model->add_statement(statement(blank('template'),
-								 $hydra->variable,
+
+	$model->add_statement(statement(blank('object'),
+								 iri($hydra->property),
+								 iri($rdf->object)));
+	$model->add_statement(statement(blank('object'),
+								 iri($hydra->variable),
 								 literal('object')));
 	$model->end_bulk_ops;
 	return $model;
@@ -867,7 +869,7 @@ Copyright 2010 Gregory Todd Williams
 
 Copyright 2010 ABC Startsiden AS
 
-Copyright 2010, 2011, 2012, 2013, 2014 Kjetil Kjernsmo
+Copyright 2010, 2011, 2012, 2013, 2014, 2015, 2016 Kjetil Kjernsmo
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
